from django.contrib import admin
from django.contrib.auth.models import User
from django.urls import reverse
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _

from .models import TelegramUser, TelegramUpdate


def linkify(field_name):
    """
    Converts a foreign key value into clickable links.
    """

    def _linkify(obj):
        linked_obj = getattr(obj, field_name)
        if linked_obj is None:
            return "Disconnected!"
        app_label = linked_obj._meta.app_label
        model_name = linked_obj._meta.model_name
        view_name = f"admin:{app_label}_{model_name}_change"
        link_url = reverse(view_name, args=[linked_obj.pk])
        return format_html('<a href="{}">{}</a>', link_url, linked_obj)

    _linkify.short_description = field_name  # Sets column name
    return _linkify


@admin.register(TelegramUser)
class Profile(admin.ModelAdmin):
    list_display = ["id", "username", "full_name", "is_bot", linkify("site_user")]
    list_display_links = ["id", "username"]
    list_filter = ["is_bot", "language_code", "date_met", "last_seen", "bots"]
    readonly_fields = ["id", "bots", "username", "first_name", "last_name", "date_met", "last_seen"]
    search_fields = [
        # Telegram fields
        "username",
        "first_name",
        "last_name",
        "bots",
        # User fields
        "site_user__first_name",
        "site_user__last_name",
        "site_user__{}".format(User.USERNAME_FIELD),
    ]
    fieldsets = (
        (
            _("personal information"),
            {
                "fields": (
                    "username",
                    ("first_name", "last_name"),
                    "phone_number",
                )
            },
        ),
        (_("Telegram data"), {"fields": ("is_bot", "language_code", "bots")}),
        (
            _("Last updates"),
            {"fields": ("date_met", "last_login")},
        ),
    )


@admin.register(TelegramUpdate)
class UpdateAdmin(admin.ModelAdmin):
    list_display = ["update_id", "bot"]
    list_display_links = ["update_id"]
    list_filter = ["bot"]
    search_fields = ["update_id", "bot", "message"]



import json
import logging
import sys

import telegram
from django.conf import settings
from django.contrib.admin.views.decorators import staff_member_required
from django.http import JsonResponse
from django.shortcuts import render
from django.views.decorators.csrf import csrf_exempt
from telegram.error import TelegramError

from .apps import DjangoTelegramBot

# Get an instance of a logger
logger = logging.getLogger(__name__)


@staff_member_required
def admin_page(request):
    bot_list = [bot.instance for bot in DjangoTelegramBot.bots_data]
    context = {"bot_list": bot_list, "update_mode": settings.DJANGO_TELEGRAMBOT.get("MODE", "WEBHOOK")}
    return render(request, "itelegram/index.html", context)


@csrf_exempt
def webhook(request, bot_token):
    # check the validity of the token
    bot = DjangoTelegramBot.getBot(bot_id=bot_token, safe=False)
    if bot is None:
        logger.warn("Request for not found token: {}".format(bot_token))
        return JsonResponse({})

    try:
        data = json.loads(request.body.decode("utf-8"))

    except:
        logger.warn("Telegram bot <{}> receive invalid request: {}".format(bot.username, repr(request)))
        return JsonResponse({})

    dispatcher = DjangoTelegramBot.getDispatcher(bot_token, safe=False)
    if dispatcher is None:
        logger.error("Dispatcher for bot <{}> not found: {}".format(bot.username, bot_token))
        return JsonResponse({})

    try:
        update = telegram.Update.de_json(data, bot)
        dispatcher.process_update(update)
        logger.debug("Bot <{}> : Processed update {}".format(bot.username, update))
    # Dispatch any errors
    except TelegramError as te:
        logger.warn("Bot <{}> : Error was raised while processing Update.".format(bot.username))
        dispatcher.dispatchError(update, te)

    # All other errors should not stop the thread, just print them
    except:
        logger.error(
            "Bot <{}> : An uncaught error was raised while processing an update\n{}".format(
                bot.username, sys.exc_info()[0]
            )
        )

    finally:
        return JsonResponse({})



from django.conf import settings
from django.contrib.postgres.fields import ArrayField
from django.db import models
from django.utils.translation import gettext_lazy as _

from .managers import TelegramUserManager


class TelegramUser(models.Model):
    objects = TelegramUserManager

    id = models.BigIntegerField(
        verbose_name=_("Telegram User ID"),
        primary_key=True,
        help_text=_("Telegram ID is a unique ID for each user which is help us tp identify users."),
    )
    first_name = models.CharField(
        verbose_name=_("Telegram User First name"),
        max_length=255,
        null=True,
        blank=True,
    )
    last_name = models.CharField(
        verbose_name=_("Telegram User Last name"),
        max_length=255,
        null=True,
        blank=True,
    )
    username = models.CharField(
        verbose_name=_("Telegram User Username"),
        max_length=255,
        null=True,
        blank=True,
    )
    is_bot = models.BooleanField(
        verbose_name=_("Telegram User IsBot"),
        default=False,
        help_text=_("Designates whether the account is a telegram bot or a human user."),
    )
    language_code = models.CharField(
        verbose_name=_("Telegram User Language code"),
        max_length=8,
        null=True,
        blank=True,
    )
    bots = ArrayField(
        models.CharField(
            verbose_name=_("Telegram bot"),
            max_length=32,
            help_text=_("Displays the bots which have been interacted with this user."),
        )
    )
    date_met = models.DateTimeField(auto_now_add=True)
    last_seen = models.DateTimeField(null=True, blank=True)

    # Optional fields
    site_user = models.OneToOneField(
        verbose_name=_("Connected Site User"),
        on_delete=models.SET_NULL,
        blank=True,
        null=True,
        related_name="telegram",
        to=settings.AUTH_USER_MODEL,
    )
    phone_number = models.CharField(
        verbose_name=_("Phone number"),
        max_length=16,
        unique=True,
    )

    class Meta:
        verbose_name = _("Telegram User")
        verbose_name_plural = _("Telegram Users")

    def __str__(self):
        return "{} {} (@{})".format(self.first_name, self.last_name, self.username)

    @property
    def full_name(self):
        return "{} {}".format(self.first_name, self.last_name).strip()


class TelegramUpdate(models.Model):
    update_id = models.CharField(
        verbose_name=_("Telegram Update ID"),
        max_length=128,
        help_text=_("This is a unique ID for each Update from Telegram"),
    )
    bot = models.CharField(
        verbose_name=_("Telegram bot"),
        max_length=32,
        help_text=_("Displays the bot which has participated is in this update."),
    )
    message = models.JSONField(
        verbose_name=_("Telegram Update message"),
        null=True,
        blank=True,
    )
    edited_message = models.JSONField(
        verbose_name=_("Telegram Update edited message"),
        null=True,
        blank=True,
    )
    channel_post = models.JSONField(
        verbose_name=_("Telegram Update channel post"),
        null=True,
        blank=True,
    )
    edited_channel_post = models.JSONField(
        verbose_name=_("Telegram Update edited channel post"),
        null=True,
        blank=True,
    )
    inline_query = models.JSONField(
        verbose_name=_("Telegram Update inline query"),
        null=True,
        blank=True,
    )
    chosen_inline_result = models.JSONField(
        verbose_name=_("Telegram Update chosen inline result"),
        null=True,
        blank=True,
    )
    callback_query = models.JSONField(
        verbose_name=_("Telegram Update callback query"),
        null=True,
        blank=True,
    )

    class Meta:
        verbose_name = _("Telegram Update")
        verbose_name_plural = _("Telegram Updates")

    def __str__(self):
        return "{} | {}".format(self.update_id, self.bot)



import importlib
import logging
import os.path
from time import sleep
from typing import List

import telegram
from django.apps import AppConfig
from django.apps import apps
from django.conf import settings
from django.utils.module_loading import module_has_submodule
from django.utils.translation import ugettext_lazy as _
from telegram.error import InvalidToken, RetryAfter, TelegramError
from telegram.ext import Dispatcher, Updater, messagequeue as mq
from telegram.utils.request import Request

from .bot import BotData
from .mqbot import MQBot

logger = logging.getLogger(__name__)

TELEGRAM_BOT_MODULE_NAME = settings.DJANGO_TELEGRAMBOT.get("BOT_MODULE_NAME", "telegrambot")
WEBHOOK_MODE, POLLING_MODE = range(2)


class classproperty(property):
    def __get__(self, obj, objtype=None):
        return super(classproperty, self).__get__(objtype)

    def __set__(self, obj, value):
        super(classproperty, self).__set__(type(obj), value)

    def __delete__(self, obj):
        super(classproperty, self).__delete__(type(obj))


class DjangoTelegramBot(AppConfig):
    name = "itelegram"
    verbose_name = _("iTelegram")
    ready_run = False
    bots_data: List[BotData] = list()
    __used_tokens = set()

    @classproperty
    def dispatcher(cls):
        try:
            # print("Getting value default dispatcher")
            bot_data = cls.bots_data[0]
            cls.__used_tokens.add(bot_data.token)
            return bot_data.dispatcher
        except (StopIteration, IndexError):
            raise ReferenceError("No bots are defined")

    @classproperty
    def updater(cls):
        try:
            # print("Getting value default dispatcher")
            bot_data = cls.bots_data[0]
            cls.__used_tokens.add(bot_data.token)
            return bot_data.updater
        except (StopIteration, IndexError):
            raise ReferenceError("No bots are defined")

    @classmethod
    def _get_bot_by_id(cls, bot_id=None, safe=True):
        if bot_id is None:
            try:
                return cls.bots_data[0]
            except IndexError:
                return None
        else:
            try:
                bot = next(filter(lambda bot_data: bot_data.token == bot_id, cls.bots_data))
            except StopIteration:
                if not safe:
                    return None
                try:
                    bot = next(filter(lambda bot_data: bot_data.unique_id == bot_id, cls.bots_data))
                except StopIteration:
                    try:
                        bot = next(filter(lambda bot_data: bot_data.instance.username == bot_id, cls.bots_data))
                    except StopIteration:
                        return None
            cls.__used_tokens.add(bot.token)
            return bot

    @classmethod
    def get_dispatcher(cls, bot_id=None, safe=True):
        bot = cls._get_bot_by_id(bot_id, safe)
        if bot:
            return bot.dispatcher
        else:
            return None

    @classmethod
    def getDispatcher(cls, bot_id=None, safe=True):
        return cls.get_dispatcher(bot_id, safe)

    @classmethod
    def get_bot(cls, bot_id=None, safe=True):
        bot = cls._get_bot_by_id(bot_id, safe)
        if bot:
            return bot.instance
        else:
            return None

    @classmethod
    def getBot(cls, bot_id=None, safe=True):
        return cls.get_bot(bot_id, safe)

    @classmethod
    def get_updater(cls, bot_id=None, safe=True):
        bot = cls._get_bot_by_id(bot_id, safe)
        if bot:
            return bot.updater
        else:
            return None

    @classmethod
    def getUpdater(cls, id=None, safe=True):
        return cls.get_updater(id, safe)

    def ready(self):
        if DjangoTelegramBot.ready_run:
            return
        DjangoTelegramBot.ready_run = True

        self.mode = WEBHOOK_MODE
        if settings.DJANGO_TELEGRAMBOT.get("MODE", "WEBHOOK") == "POLLING":
            self.mode = POLLING_MODE

        modes = ["WEBHOOK", "POLLING"]
        logger.info("Django Telegram Bot <{} mode>".format(modes[self.mode]))

        bots_list = settings.DJANGO_TELEGRAMBOT.get("BOTS", [])

        if self.mode == WEBHOOK_MODE:
            webhook_site = settings.DJANGO_TELEGRAMBOT.get("WEBHOOK_SITE", None)
            if not webhook_site:
                logger.warn("Required TELEGRAM_WEBHOOK_SITE missing in settings")
                return
            if webhook_site.endswith("/"):
                webhook_site = webhook_site[:-1]

            webhook_base = settings.DJANGO_TELEGRAMBOT.get("WEBHOOK_PREFIX", "/")
            webhook_base = webhook_base.replace("/", "")
            if len(webhook_base):
                webhook_base += "/"

            cert = settings.DJANGO_TELEGRAMBOT.get("WEBHOOK_CERTIFICATE", None)
            certificate = None
            if cert and os.path.exists(cert):
                logger.info("WEBHOOK_CERTIFICATE found in {}".format(cert))
                certificate = open(cert, "rb")
            elif cert:
                logger.error("WEBHOOK_CERTIFICATE not found in {} ".format(cert))

        for b in bots_list:
            bot = BotData(
                token=b["TOKEN"],
                unique_id=b.get("ID", None),
                use_context=b.get("CONTEXT", False),
                allowed_updates=b.get("ALLOWED_UPDATES", None),
                timeout=b.get("TIMEOUT", None),
                proxy=b.get("PROXY", None),
            )

            if self.mode == WEBHOOK_MODE:
                try:
                    if b.get("MESSAGEQUEUE_ENABLED", False):
                        q = mq.MessageQueue(
                            all_burst_limit=b.get("MESSAGEQUEUE_ALL_BURST_LIMIT", 29),
                            all_time_limit_ms=b.get("MESSAGEQUEUE_ALL_TIME_LIMIT_MS", 1024),
                        )
                        if bot.proxy:
                            request = Request(
                                proxy_url=bot.proxy["proxy_url"],
                                urllib3_proxy_kwargs=bot.proxy["urllib3_proxy_kwargs"],
                                con_pool_size=b.get("MESSAGEQUEUE_REQUEST_CON_POOL_SIZE", 8),
                            )
                        else:
                            request = Request(con_pool_size=b.get("MESSAGEQUEUE_REQUEST_CON_POOL_SIZE", 8))
                        bot.instance = MQBot(bot.token, request=request, mqueue=q)
                    else:
                        request = None
                        if bot.proxy:
                            request = Request(
                                proxy_url=bot.proxy["proxy_url"], urllib3_proxy_kwargs=bot.proxy["urllib3_proxy_kwargs"]
                            )
                        bot.instance = telegram.Bot(token=bot.token, request=request)

                    bot.dispatcher = Dispatcher(bot.instance, None, workers=0, use_context=bot.use_context)
                    if not settings.DJANGO_TELEGRAMBOT.get("DISABLE_SETUP", False):
                        hook_url = "{}/itelegram/{}{}/".format(webhook_site, webhook_base, bot.token)
                        max_connections = b.get("WEBHOOK_MAX_CONNECTIONS", 40)
                        result = bot.instance.setWebhook(
                            hook_url,
                            certificate=certificate,
                            timeout=bot.timeout,
                            max_connections=max_connections,
                            allowed_updates=bot.allowed_updates,
                        )
                        webhook_info = bot.instance.getWebhookInfo()
                        real_allowed = webhook_info.allowed_updates if webhook_info.allowed_updates else ["ALL"]
                        bot.more_info = webhook_info
                        logger.info(
                            (
                                "Telegram Bot <{bot_username}> setting webhook [ {webhook_url} ] "
                                "max connections:{webhook_max_connection} "
                                "allowed updates:{webhook_allowed_update} "
                                "pending updates:{webhook_pending_update} "
                                "result: {result}"
                            ).format(
                                bot_username=bot.instance.username,
                                webhook_url=webhook_info.url,
                                webhook_max_connection=webhook_info.max_connections,
                                webhook_allowed_update=real_allowed,
                                webhook_pending_update=webhook_info.pending_update_count,
                                result=result,
                            )
                        )
                    else:
                        logger.info("Telegram Bot setting webhook without enabling receiving")
                except InvalidToken:
                    logger.error("Invalid Token : {}".format(bot.token))
                    return
                except RetryAfter as er:
                    logger.debug(
                        'Error: "{message}". Will retry in {retry_after} seconds'.format(
                            message=er.message, retry_after=er.retry_after
                        )
                    )
                    sleep(er.retry_after)
                    self.ready()
                except TelegramError as er:
                    logger.error('Error: "{}"'.format(er.message))
                    return

            else:
                try:
                    if not settings.DJANGO_TELEGRAMBOT.get("DISABLE_SETUP", False):
                        bot.updater = Updater(token=bot.token, request_kwargs=bot.proxy, use_context=bot.use_context)
                        bot.instance = bot.updater.bot
                        bot.instance.delete_webhook()
                        bot.dispatcher = bot.updater.dispatcher
                        DjangoTelegramBot.__used_tokens.add(bot.token)
                    else:
                        request = None
                        if bot.proxy:
                            request = Request(
                                proxy_url=bot.proxy["proxy_url"], urllib3_proxy_kwargs=bot.proxy["urllib3_proxy_kwargs"]
                            )
                        bot.instance = telegram.Bot(token=bot.token, request=request)
                        bot.dispatcher = Dispatcher(bot.instance, None, workers=0, use_context=bot.use_context)
                except InvalidToken:
                    logger.error("Invalid Token : {}".format(bot.token))
                    return
                except RetryAfter as er:
                    logger.debug(
                        'Error: "{message}". Will retry in {retry_after} seconds'.format(
                            message=er.message, retry_after=er.retry_after
                        )
                    )
                    sleep(er.retry_after)
                    self.ready()
                except TelegramError as er:
                    logger.error('Error: "{}"'.format(er.message))
                    return

            DjangoTelegramBot.bots_data.append(bot)

        first_bot = DjangoTelegramBot.bots_data[0]
        if not settings.DJANGO_TELEGRAMBOT.get("DISABLE_SETUP", False):
            logger.debug("Telegram Bot <{}> set as default bot".format(first_bot.instance.username))
        else:
            logger.debug(
                "Telegram Bot <{}> set as default bot".format(
                    first_bot.unique_id if first_bot.unique_id else first_bot.token
                )
            )

        def module_imported(module_name, method_name, execute):
            try:
                m = importlib.import_module(module_name)
                if execute and hasattr(m, method_name):
                    logger.debug("Run {}.{}()".format(module_name, method_name))
                    getattr(m, method_name)()
                else:
                    logger.debug("Run {}".format(module_name))

            except ImportError as er:
                if settings.DJANGO_TELEGRAMBOT.get("STRICT_INIT"):
                    raise er
                else:
                    logger.error("{} : {}".format(module_name, repr(er)))
                    return False

            return True

        # import telegram bot handlers for all INSTALLED_APPS
        # it allows us to use the telegram bots in all installed apps!
        for app_config in apps.get_app_configs():
            if module_has_submodule(app_config.module, TELEGRAM_BOT_MODULE_NAME):
                module_name = "%s.%s" % (app_config.name, TELEGRAM_BOT_MODULE_NAME)
                if module_imported(module_name, "main", True):
                    logger.info("Loaded {}".format(module_name))

        num_bots = len(DjangoTelegramBot.__used_tokens)
        if self.mode == POLLING_MODE and num_bots > 0:
            logger.info(
                "Please manually start polling update for {0} bot{1}. Run command{1}:".format(
                    num_bots, "s" if num_bots > 1 else ""
                )
            )
            for token in DjangoTelegramBot.__used_tokens:
                updater = DjangoTelegramBot.get_updater(bot_id=token)
                logger.info("python manage.py botpolling --username={}".format(updater.bot.username))



from datetime import datetime

from django.db import models


class TelegramUserManager(models.Manager):
    """Custom Telegram User manager"""

    def get_recent_users(self, from_date: datetime):
        """
        The last users who have interaction with us
        :return QuerySet
        """
        recent_users = self.filter(date_met__gte=from_date)
        return recent_users



from django.conf import settings
from django.urls import path

from . import views

webhook_base: str = settings.DJANGO_TELEGRAMBOT.get("WEBHOOK_PREFIX", "")
# Fix  /
webhook_base = webhook_base.replace("/", "")
if len(webhook_base):
    webhook_base += "/"

urlpatterns = [
    path("admin", views.admin_page, name="itelegram"),
    path("{}<str:bot_token>".format(webhook_base), views.webhook, name="webhook"),
]



import logging

from django.core.management.base import BaseCommand

from itelegram.apps import DjangoTelegramBot


class Command(BaseCommand):
    help = "Run telegram bot in polling mode"
    can_import_settings = True

    def add_arguments(self, parser):
        parser.add_argument("--username", "-i", help="Bot username", default=None)
        parser.add_argument("--token", "-t", help="Bot token", default=None)
        pass

    def get_updater(self, username=None, token=None):
        updater = None
        if username is not None:
            updater = DjangoTelegramBot.get_updater(bot_id=username)
            if not updater:
                self.stderr.write("Cannot find default bot with username {}".format(username))
        elif token:
            updater = DjangoTelegramBot.get_updater(bot_id=token)
            if not updater:
                self.stderr.write("Cannot find bot with token {}".format(token))
        return updater

    def handle(self, *args, **options):
        from django.conf import settings

        if settings.DJANGO_TELEGRAMBOT.get("MODE", "WEBHOOK") == "WEBHOOK":
            self.stderr.write("Webhook mode active in settings.py, change in POLLING if you want use polling update")
            return

        updater = self.get_updater(username=options.get("username"), token=options.get("token"))
        if not updater:
            self.stderr.write("Bot not found")
            return
        # Enable Logging
        logging.basicConfig(format="%(asctime)s - %(name)s - %(levelname)s - %(message)s", level=logging.INFO)
        logger = logging.getLogger("telegrambot")
        logger.setLevel(logging.INFO)
        console = logging.StreamHandler()
        console.setLevel(logging.INFO)
        console.setFormatter(logging.Formatter("%(name)s - %(levelname)s - %(message)s"))
        logger.addHandler(console)

        bots_list = settings.DJANGO_TELEGRAMBOT.get("BOTS", [])
        b = None
        for bot_set in bots_list:
            if bot_set.get("TOKEN", None) == updater.bot.token:
                b = bot_set
                break
        if not b:
            self.stderr.write("Cannot find bot settings")
            return

        allowed_updates = b.get("ALLOWED_UPDATES", None)
        timeout = b.get("TIMEOUT", 10)
        poll_interval = b.get("POLL_INTERVAL", 0.0)
        clean = b.get("POLL_CLEAN", False)
        bootstrap_retries = b.get("POLL_BOOTSTRAP_RETRIES", 0)
        read_latency = b.get("POLL_READ_LATENCY", 2.0)

        self.stdout.write("Run polling...")
        updater.start_polling(
            poll_interval=poll_interval,
            timeout=timeout,
            clean=clean,
            bootstrap_retries=bootstrap_retries,
            read_latency=read_latency,
            allowed_updates=allowed_updates,
        )
        self.stdout.write("the bot is started and runs until we press Ctrl-C on the command line.")
        updater.idle()



import logging

from django.conf import settings
from django.utils.timezone import now
from telegram import Update, TelegramError
from telegram.ext import CallbackContext

from .defaults import telegram_internal_error_text, telegram_parse_mode
from .models import TelegramUpdate, TelegramUser

logger = logging.getLogger(__name__)


def process_error(update: Update, context: CallbackContext):
    logger.error("Internal Error: {}".format(context.error))
    try:
        context.bot.send_message(
            chat_id=update.message.from_user.id,
            text=getattr(settings, "TELEGRAM_INTERNAL_ERROR_TEXT", telegram_internal_error_text),
            parse_mode=getattr(settings, "TELEGRAM_PARSE_MODE", telegram_parse_mode),
        )
    except TelegramError as error:
        logger.error("Error: {message}".format(message=error))
        pass
    raise context.error


def process_update(update: Update, bot: str):
    def get_message_users(message):
        users = [message.get("from", None), message.get("forward_from", None), message.get("left_chat_member", None)]
        for user in message.get("new_chat_members", []):
            users.append(user)

        if message.get("reply_to_message", None):
            users += get_message_users(message["reply_to_message"])

        if message.get("pinned_message", None):
            users += get_message_users(message["pinned_message"])

        return users

    users = []
    if update.message:
        users += get_message_users(update.message.to_dict())

    if update.edited_message:
        users += get_message_users(update.edited_message.to_dict())

    users = [user for user in users if user is not None and user.get("id", None) is not None]  # Remove None values
    users = list({user["id"]: user for user in users}.values())  # Remove duplicate users

    for user in users:
        try:
            current_user = TelegramUser.objects.get(id=user["id"])
            TelegramUser.objects.filter(id=user["id"]).update(
                first_name=user["first_name"],
                last_name=user.get("last_name", None),
                is_bot=user["is_bot"],
                username=user.get("username", None),
                language_code=user.get("language_code", None),
                bots=list(set(current_user.bots + [bot])),
                last_seen=now(),
            )
        except TelegramUser.DoesNotExist:
            TelegramUser.objects.update_or_create(
                id=user["id"],
                first_name=user["first_name"],
                last_name=user.get("last_name", None),
                is_bot=user["is_bot"],
                username=user.get("username", None),
                language_code=user.get("language_code", None),
                bots=[bot],
                last_seen=now(),
            )

    TelegramUpdate.objects.create(
        update_id=update.update_id,
        bot=bot,
        message=update.message.to_json() if update.message else None,
        edited_message=update.edited_message.to_json() if update.edited_message else None,
        channel_post=update.channel_post.to_json() if update.channel_post else None,
        edited_channel_post=update.edited_channel_post.to_json() if update.edited_channel_post else None,
        inline_query=update.inline_query.to_json() if update.inline_query else None,
        chosen_inline_result=update.chosen_inline_result.to_json() if update.chosen_inline_result else None,
        callback_query=update.callback_query.to_json() if update.callback_query else None,
    )



from functools import wraps

from django.conf import settings
from telegram import Update
from telegram.ext import CallbackContext, DispatcherHandlerStop

from .defaults import telegram_not_logged_in_text, telegram_permission_denied_text, telegram_parse_mode
from .models import TelegramUser


def user_passes_test(test_func):
    def decorator(view_func):
        @wraps(view_func)
        def _wrapped_view(update: Update, context: CallbackContext, *args, **kwargs):
            chat_id = update.message.from_user.id
            try:
                user = TelegramUser.objects.get(id=chat_id)
                if not user.site_user:
                    raise TelegramUser.DoesNotExist
            except TelegramUser.DoesNotExist:
                context.bot.send_message(
                    chat_id=chat_id,
                    text=getattr(settings, "TELEGRAM_NOT_LOGGED_IN_TEXT", telegram_not_logged_in_text),
                    parse_mode=getattr(settings, "TELEGRAM_PARSE_MODE", telegram_parse_mode),
                )
                raise DispatcherHandlerStop
            if test_func(user.site_user):
                return view_func(update, context, *args, **kwargs)
            context.bot.send_message(
                chat_id=chat_id,
                text=getattr(settings, "TELEGRAM_PERMISSION_DENIED_TEXT", telegram_permission_denied_text),
                parse_mode=getattr(settings, "TELEGRAM_PARSE_MODE", telegram_parse_mode),
            )
            raise DispatcherHandlerStop

        return _wrapped_view

    return decorator


def telegram_perm(perm):
    def check_perms(user):
        if isinstance(perm, str):
            perms = (perm,)
        else:
            perms = perm
        # Check if the user has the permission
        if user.has_perms(perms):
            return True
        # User has not permissions
        return False

    return user_passes_test(check_perms)



# Generated by Django 3.1.2 on 2020-10-24 15:47

import django.contrib.postgres.fields
import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="TelegramUpdate",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "update_id",
                    models.CharField(
                        help_text="This is a unique ID for each Update from Telegram",
                        max_length=128,
                        verbose_name="Telegram Update ID",
                    ),
                ),
                (
                    "bot",
                    models.CharField(
                        help_text="Displays the bot which has participated is in this update.",
                        max_length=32,
                        verbose_name="Telegram bot",
                    ),
                ),
                ("message", models.JSONField(blank=True, null=True, verbose_name="Telegram Update message")),
                (
                    "edited_message",
                    models.JSONField(blank=True, null=True, verbose_name="Telegram Update edited message"),
                ),
                ("channel_post", models.JSONField(blank=True, null=True, verbose_name="Telegram Update channel post")),
                (
                    "edited_channel_post",
                    models.JSONField(blank=True, null=True, verbose_name="Telegram Update edited channel post"),
                ),
                ("inline_query", models.JSONField(blank=True, null=True, verbose_name="Telegram Update inline query")),
                (
                    "chosen_inline_result",
                    models.JSONField(blank=True, null=True, verbose_name="Telegram Update chosen inline result"),
                ),
                (
                    "callback_query",
                    models.JSONField(blank=True, null=True, verbose_name="Telegram Update callback query"),
                ),
            ],
            options={
                "verbose_name": "Telegram Update",
                "verbose_name_plural": "Telegram Updates",
            },
        ),
        migrations.CreateModel(
            name="TelegramUser",
            fields=[
                (
                    "id",
                    models.BigIntegerField(
                        help_text="Telegram ID is a unique ID for each user which is help us tp identify users.",
                        primary_key=True,
                        serialize=False,
                        verbose_name="Telegram User ID",
                    ),
                ),
                (
                    "first_name",
                    models.CharField(blank=True, max_length=255, null=True, verbose_name="Telegram User First name"),
                ),
                (
                    "last_name",
                    models.CharField(blank=True, max_length=255, null=True, verbose_name="Telegram User Last name"),
                ),
                (
                    "username",
                    models.CharField(blank=True, max_length=255, null=True, verbose_name="Telegram User Username"),
                ),
                (
                    "is_bot",
                    models.BooleanField(
                        default=False,
                        help_text="Designates whether the account is a telegram bot or a human user.",
                        verbose_name="Telegram User IsBot",
                    ),
                ),
                (
                    "language_code",
                    models.CharField(blank=True, max_length=8, null=True, verbose_name="Telegram User Language code"),
                ),
                (
                    "bots",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.CharField(
                            help_text="Displays the bots which have been interacted with this user.",
                            max_length=32,
                            verbose_name="Telegram bot",
                        ),
                        size=None,
                    ),
                ),
                ("date_met", models.DateTimeField(auto_now_add=True)),
                ("last_seen", models.DateTimeField(blank=True, null=True)),
                ("phone_number", models.CharField(max_length=16, unique=True, verbose_name="Phone number")),
                (
                    "site_user",
                    models.OneToOneField(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="telegram",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="Connected Site User",
                    ),
                ),
            ],
            options={
                "verbose_name": "Telegram User",
                "verbose_name_plural": "Telegram Users",
            },
        ),
    ]



telegram_not_logged_in_text = (
    "*You're not logged  in!*\n"
    "To access this section you should login first\n"
    "It's probably possible by \login command"
)

telegram_permission_denied_text = (
    "*Permission Denied!*\n"
    "You don't have access for this section yet\n"
    "make sure you have the correct access and for more contact admin"
)

telegram_internal_error_text = (
    "*Internal Error*\n"
    "We have encountered an internal error!\n"
    "Our team is notified and is working on this problem.\n"
    "Sorry for the inconvenience."
)

telegram_parse_mode = "Markdown"



import telegram
from telegram.ext import messagequeue as mq


class MQBot(telegram.bot.Bot):
    """A subclass of Bot which delegates send method handling to MQ"""

    def __init__(self, mqueue=None, is_queued_def=True, *args, **kwargs):
        super(MQBot, self).__init__(*args, **kwargs)
        # below 2 attributes should be provided for decorator usage
        self._is_messages_queued_default = is_queued_def
        self._msg_queue = mqueue or mq.MessageQueue()

    def __del__(self):
        try:
            self._msg_queue.stop()
        except:
            pass
        super(MQBot, self).__del__()

    @mq.queuedmessage
    def send_message(self, *args, **kwargs):
        """Wrapped method would accept new `queued` and `isgroup`
        OPTIONAL arguments"""
        return super(MQBot, self).send_message(*args, **kwargs)

    @mq.queuedmessage
    def edit_message_text(self, *args, **kwargs):

        """Wrapped method would accept new `queued` and `isgroup`
        OPTIONAL arguments"""
        return super(MQBot, self).edit_message_text(*args, **kwargs)



from telegram import Bot
from telegram.ext import Dispatcher, Updater


class BotData:
    def __init__(
        self,
        token,
        *,
        unique_id=None,
        use_context=False,
        allowed_updates=None,
        timeout=None,
        proxy=None,
        instance: Bot = None,
        dispatcher: Dispatcher = None,
        updater: Updater = None
    ):
        self.token = token
        self.unique_id = unique_id
        self.use_context = use_context
        self.allowed_updates = allowed_updates
        self.timeout = timeout
        self.proxy = proxy
        self.instance = instance
        self.dispatcher = dispatcher
        self.updater = updater
